# -*- coding: utf-8 -*- 

import random
import pygame
from pygame.locals import *

class figurasGeometricas():
	RED = (255,0,0)
	BLACK = (0, 0, 0, 0)
	WHITE = (255, 255, 255)
	DARK_SLATE_GRAY = (47, 79, 79)
	NAVY_BLUE = (00, 00, 80)
	DARK_OLIVE_GREE = (85, 107, 47)
	GOLDENROD = (218, 165, 32)

	def __init__(self):
		pygame.init()
		self.fonte = pygame.font.Font(None, 24)
		pygame.display.set_caption('Computacao Grafica - Figuras Geometricas')
		self.screen = pygame.display.set_mode((640, 480))
		self.menu()
		
		
	def run(self):
		self.loop()
		pygame.quit()

	def loop(self):
		exit = False
		while not exit:
			exit = self.handleEvents()
			pygame.display.update()

	def handleEvents(self):
		exit = False
		for event in pygame.event.get():
				if event.type == QUIT:
						exit = True
				elif event.type == KEYDOWN:
						if event.key == K_ESCAPE:
								exit = True
				elif event.type == MOUSEBUTTONDOWN:
						self.handleMouseDown(pygame.mouse.get_pos())
		return exit

	def handleMouseDown(self, (x, y)):
		if (self.botaoRetangulo.collidepoint(x, y)):
			self.figuraAtual = "RETANGULO"
			self.coordenadas = []
			self.pontos = 0
		elif (self.botaoCirculo.collidepoint(x, y)):
			self.figuraAtual = "CIRCULO"
		elif (self.botaoLimparTela.collidepoint(x, y)):
			self.menu()
		elif (self.botaoPoligono.collidepoint(x, y)):
			self.figuraAtual = "POLIGONO"
			self.coordenadas = []
			self.pontos = 0
		
		if (self.fundoMenu.collidepoint(x, y)):		# se clicar no menu, ele retorno, pois nao pode desenhar em cima dele
			return
		
		if self.figuraAtual == "CIRCULO":
			if y < 130:		# para previnir que desenhe um circulo no menu
				y += 30
			self.desenharCirculo(x, y, 30, 0)
			
		elif self.figuraAtual == "RETANGULO":
			self.desenharCirculo(x, y, radius=2, width=1, cor=self.BLACK)
			ponto = (x, y)
			self.coordenadas.append((x, y))
			self.pontos += 1			
			if self.pontos == 2:
				self.pontos = 0
				left = self.coordenadas[0][0]
				top = self.coordenadas[0][1]
				width = self.coordenadas[1][0] - left
				height = self.coordenadas[1][1] - top
				self.coordenadas = []
				retangulo = self.desenharRetangulo(left, top, width, height)
				for j in self.retangulos:
					if j.colliderect(retangulo):
						pygame.draw.rect(self.screen, self.RED, j.clip(retangulo))
				self.retangulos.append(retangulo)

		elif self.figuraAtual == "POLIGONO":
			self.desenharCirculo(x, y, radius=2, width=1, cor=self.BLACK)
			self.coordenadas.append((x, y))
			self.pontos += 1
			if self.pontos == 4:
				poligono = self.desenharPoligono(self.coordenadas, 0)
				for j in self.poligonos:
					if j.colliderect(poligono):
						pygame.draw.rect(self.screen, self.RED, j.clip(poligono))
				self.poligonos.append(poligono)
				self.coordenadas = []
				self.pontos = 0
		

	def menu(self):
		self.screen.fill(self.WHITE)
		self.ultimoRetangulo = None
		self.circulos = []
		self.poligonos = []
		self.coordenadas = []
		self.retangulos = []
		self.pontos = 0
		self.figuraAtual = "RETANGULO"
		self.fundoMenu = self.desenharRetangulo(0, 0, 640, 100, (230, 230, 230))
		self.botaoRetangulo = self.desenharRetangulo(10, 10, 100, 60, self.GOLDENROD)
		self.botaoCirculo = self.desenharCirculo(165, 40, 30, 0, self.NAVY_BLUE)
		self.botaoPoligono = self.desenharPoligono(((220, 70), (240, 10), (320, 20), (340, 70)), 0, self.DARK_SLATE_GRAY)
		self.botaoLimparTela = self.desenharRetangulo(540, 10, 100, 30, self.DARK_OLIVE_GREE)
		self.textoRetangulo = self.escrever("Retngulo", (20, 80))
		self.textoCirculo = self.escrever("Circulo", (135, 80))
		self.textoPoligono = self.escrever("Quadriltero", (230, 80))
		self.textoBotaoLimpar = self.escrever("Limpar tela", (545, 20))


	def escrever(self, texto, posicao, cor=None):
		if cor == None:
			cor = self.BLACK
			
		img = self.fonte.render(texto, True, cor)
		self.screen.blit(img, posicao)
		pygame.display.flip()
		return img
		
		
	def desenharCirculo(self, x, y, radius, width, cor=None):
		if cor == None:
			cor = (random.randrange(0, 225, 25), random.randrange(0, 225, 25), random.randrange(0, 225, 25))
		return pygame.draw.circle(
			self.screen,
			cor,
			(x, y),
			radius,
			width
		)
			
	def desenharLinha(self, inicio, fim, cor=None):
		if cor == None:
			cor = (random.randrange(0, 225, 25), random.randrange(0, 225, 25), random.randrange(0, 225, 25))
		return pygame.draw.line(
			self.screen, 
			self.BLACK, 
			inicio, 
			fim, 
			1
		)


	def desenharPoligono(self, pontos, width=0, cor=None):
		if cor == None:
			cor = (random.randrange(0, 225, 25), random.randrange(0, 225, 25), random.randrange(0, 225, 25))
		return pygame.draw.polygon(
			self.screen,
			cor,
			pontos,
			width
		)
		
	def desenharRetangulo(self, left, top, width, height, cor=None):
		if cor == None:
			cor = (random.randrange(0, 225, 25), random.randrange(0, 225, 25), random.randrange(0, 225, 25))
		rect = pygame.draw.rect(
			self.screen,
			cor,
			(left, top, width, height),
		)
		pygame.display.update()
		return rect
		
figurasGeometricas().run()
